#include <fggl/gui/containers.hpp>
#include <spdlog/spdlog.h>

namespace fggl::gui {

	Widget *Container::getChildAt(const math::vec2 &point) {
		for ( auto& child : m_children ){
			if ( child->contains(point) ){
				return child->getChildAt(point);
			}
		}

		return nullptr;
	}

	bool Container::contains(const math::vec2 &point) {
		return true;
	}

	void Container::render(gfx::Paint &paint) {
		for( auto& child : m_children ){
			child->render( paint );
		}
	}

	void Container::add(std::unique_ptr<Widget> widget) {
		m_children.push_back( std::move(widget) );
		m_dirty = true;
	}

	/*
	Box::Box( LayoutAxis axis ) : m_axis( axis ) {}
	
	void Box::layout() {
		math::vec2 baseSize{};
		
		const int mainAxis = 0;
		const int crossAxis = 1;
		
		// ask for preferred sizes
		for ( auto& child : m_children ) {
			auto childBaseSize = child.baseSize();
			
			baseSize[mainAxis] += childBaseSize[mainAxis];
			if ( childBaseSize[crossAxis] > baseSize[crossAxis] ) {
				baseSize[crossAxis] = childBaseSize[crossAxis];			
			}
		}
		
		if ( lineAxis == 0 )
			setSize(lineSum, biggestCross);
		else
			setSize(biggestCross, lineSum);
	}*/

	void Panel::render(gfx::Paint &paint) {
		spdlog::info("Imma render mah panel! {}, {}", bottomRight().x, bottomRight().y );

		// background painting time
		gfx::Path2D background(topLeft());
		background.colour(math::vec3(1.0F, 1.0F, 0.0F));
		draw_box(background, topLeft(), bottomRight());
		paint.fill(background);

		// call parent's render method for child objects
		Container::render(paint);
	}
};
