#ifndef FGGL_GUI_WIDGET_H
#define FGGL_GUI_WIDGET_H

#include <fggl/math/types.hpp>
#include <fggl/gfx/paint.hpp>

#include <memory>

namespace fggl::gui {

	void draw_box( gfx::Path2D& path, math::vec2 topLeft, math::vec2 bottomRight);
	void draw_progress( gfx::Path2D& path, math::vec2 topLeft, math::vec2 size, float value);
	void draw_slider( gfx::Path2D& path, math::vec2 topLeft, math::vec2 size, float value);
	void draw_button( gfx::Path2D& path, math::vec2 topLeft, math::vec2 size, bool active, bool pressed);

	struct Bounds2D {
		math::vec2 topLeft;
		math::vec2 size;

		Bounds2D() = default;
		inline Bounds2D(math::vec2 pos, math::vec2 a_size) : topLeft(pos), size(a_size) {}

		inline bool contains(math::vec2 point) {
			return ! ( (point.x > topLeft.x + size.x) ||
			           (point.x < topLeft.x ) ||
					   (point.y > topLeft.y + size.y) ||
				       (point.y < topLeft.y)
					   );
		}
	};

	class Widget {
		public:
			Widget() = default;
			inline Widget(math::vec2 pos, math::vec2 size) : m_bounds(pos, size) {}

			virtual ~Widget() = default;

			inline math::vec2 topLeft() const {
				return m_bounds.topLeft;
			};

			inline math::vec2 bottomRight() const {
				return m_bounds.topLeft + m_bounds.size;
			}

			inline void size(math::vec2 pos, math::vec2 size) {
				m_bounds.topLeft = pos;
				m_bounds.size = size;
			}

			inline math::vec2 size() const {
				return m_bounds.size;
			}

			virtual inline bool contains(const math::vec2 &point){
				return m_bounds.contains(point);
			};

			virtual inline Widget *getChildAt(const math::vec2 &point) {
				if ( !contains(point) ) {
					return nullptr;
				}
				return this;
			}

			virtual void render(gfx::Paint &paint) = 0;
			inline virtual void activate() {};

			inline virtual void onEnter() {}
			inline virtual void onExit() {}

		private:
			Bounds2D m_bounds;
	};


}; //namespace fggl::gui

#endif
